@@ -388,7 +388,7 @@ public class FurnitureLibraryFileRecorder implements FurnitureLibraryRecorder {
       if (contentMatchingFurnitureName
           || !(pieceModel instanceof URLContent)
           || ((URLContent)pieceModel).getURL().getFile().toString().endsWith("model.obj")) {
-        contentBaseName = piece.getName();
+        contentBaseName = piece.getName().replace('/', '-');
       } else {
         String file = ((URLContent)pieceModel).getURL().getFile();
         if (file.lastIndexOf('/') != -1) {
@@ -448,6 +448,10 @@ public class FurnitureLibraryFileRecorder implements FurnitureLibraryRecorder {
       writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.WIDTH, i, piece.getWidth());
       writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.DEPTH, i, piece.getDepth());
       writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.HEIGHT, i, piece.getHeight());
+      if (Math.abs(piece.getDropOnTopElevation() - 1f) > 1E-6f) {
+        writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.DROP_ON_TOP_ELEVATION, i, 
+            piece.getDropOnTopElevation() * piece.getHeight());
+      }
       writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.MOVABLE, i, piece.isMovable());
       writeProperty(writer, DefaultFurnitureCatalog.PropertyKey.DOOR_OR_WINDOW, i, piece.isDoorOrWindow());
       if (piece.isDoorOrWindow()) {
@@ -516,7 +520,7 @@ public class FurnitureLibraryFileRecorder implements FurnitureLibraryRecorder {
             lightSourceX += DECIMAL_FORMAT.format(lightSources [lightIndex].getX() * light.getWidth());
             lightSourceY += DECIMAL_FORMAT.format(lightSources [lightIndex].getY() * light.getDepth());
             lightSourceZ += DECIMAL_FORMAT.format(lightSources [lightIndex].getZ() * light.getHeight());
-            lightSourceColor += "#" + Integer.toHexString(lightSources [lightIndex].getColor());
+            lightSourceColor += "#" + String.format("%06X", lightSources [lightIndex].getColor());
             if (lightSources [lightIndex].getDiameter() != null) {
               if (lightSourceDiameter == null) {
                 lightSourceDiameter = "";
