@@ -44,6 +44,7 @@ import com.eteks.sweethome3d.model.Content;
 import com.eteks.sweethome3d.model.FurnitureCatalog;
 import com.eteks.sweethome3d.model.FurnitureCategory;
 import com.eteks.sweethome3d.model.Sash;
+import com.eteks.sweethome3d.viewcontroller.ContentManager;
 import com.eteks.sweethome3d.viewcontroller.Controller;
 import com.eteks.sweethome3d.viewcontroller.DialogView;
 import com.eteks.sweethome3d.viewcontroller.View;
@@ -73,6 +74,7 @@ public class FurnitureController implements Controller {
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_CREATION_DATE_PROPERTY, Property.CREATION_DATE);
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_CATEGORY_PROPERTY, Property.CATEGORY);
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_CREATOR_PROPERTY, Property.CREATOR);
+    PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_MODEL_PROPERTY, Property.MODEL);
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_ICON_PROPERTY, Property.ICON);
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_WIDTH_PROPERTY, Property.WIDTH);
     PROPERTIES_MAP.put(FurnitureLibrary.FURNITURE_DEPTH_PROPERTY, Property.DEPTH);
@@ -98,6 +100,7 @@ public class FurnitureController implements Controller {
   private final FurnitureLibraryUserPreferences preferences;
   private final FurnitureLanguageController     furnitureLanguageController;
   private final EditorViewFactory               viewFactory;
+  private final ContentManager                  contentManager;
   private final PropertyChangeSupport           propertyChangeSupport;
   private DialogView                            homeFurnitureView;
 
@@ -145,12 +148,14 @@ public class FurnitureController implements Controller {
                              List<CatalogPieceOfFurniture> modifiedFurniture,
                              FurnitureLibraryUserPreferences preferences, 
                              FurnitureLanguageController furnitureLanguageController,
-                             EditorViewFactory viewFactory) {
+                             EditorViewFactory viewFactory,
+                             ContentManager    contentManager) {
     this.furnitureLibrary = furnitureLibrary;
     this.modifiedFurniture = modifiedFurniture;
     this.preferences = preferences;
     this.furnitureLanguageController = furnitureLanguageController;
     this.viewFactory = viewFactory;
+    this.contentManager = contentManager;
     this.propertyChangeSupport = new PropertyChangeSupport(this);
     
     this.editableProperties = new HashSet<Property>();
@@ -253,6 +258,13 @@ public class FurnitureController implements Controller {
     addPropertyChangeListener(Property.HEIGHT, this.heightChangeListener);
   }
 
+  /**
+   * Returns the content manager of this controller.
+   */
+  public ContentManager getContentManager() {
+    return this.contentManager;
+  }
+  
   /**
    * Returns <code>true</code> if the given <code>property</code> is editable.
    * Depending on whether a property is editable or not, the view associated to this controller
@@ -264,6 +276,7 @@ public class FurnitureController implements Controller {
     } else {
       return this.editableProperties.contains(property)
           && property != Property.ID
+          && property != Property.MODEL
           && property != Property.ICON
           && property != Property.DOOR_OR_WINDOW_CUT_OUT_SHAPE;
     }
@@ -278,7 +291,7 @@ public class FurnitureController implements Controller {
       setName(null);
       setDescription(null);
       setCategory(null);
-      setModel(null);
+      setModel((Content)null);
       setIcon(null);
       setWidth(null);
       setDepth(null);
@@ -308,7 +321,7 @@ public class FurnitureController implements Controller {
         this.editableProperties.add(Property.BACK_FACE_SHOWN);
       } else {
         setIcon(null);
-        setModel(null);
+        setModel((Content)null);
         this.editableProperties.remove(Property.BACK_FACE_SHOWN);
       }
       
@@ -816,6 +829,32 @@ public class FurnitureController implements Controller {
     return this.model;
   }
 
+  /**
+   * Reads the 3D model from the given parameter and updates the model if it's valid.
+   */
+  public void setModel(String modelName) {
+    final FurnitureLibrary furnitureLibrary = new FurnitureLibrary();
+    Runnable postImportTask = new Runnable() {
+        public void run() {
+          List<CatalogPieceOfFurniture> furniture = furnitureLibrary.getFurniture();
+          if (!furniture.isEmpty()) {
+            CatalogPieceOfFurniture piece = furniture.get(0);
+            setModel(piece.getModel());
+            setModelRotation(piece.getModelRotation());
+            // Reset object size 
+            boolean proportional = isProportional();
+            setProportional(false);
+            setWidth(piece.getWidth());
+            setDepth(piece.getDepth());
+            setHeight(piece.getHeight());
+            setProportional(proportional);
+          }
+        }
+      };
+    new ImportFurnitureController(furnitureLibrary, new String [] {modelName}, postImportTask, 
+        this.preferences, this.viewFactory, this.contentManager).executeTask(getView());
+  }
+
   /**
    * Sets the edited width.
    */
@@ -1246,6 +1285,7 @@ public class FurnitureController implements Controller {
         float pieceWidth = piece.getWidth();
         float pieceDepth = piece.getDepth();
         float pieceHeight = piece.getHeight();
+        float pieceDropOnTopElevation = piece.getDropOnTopElevation();
         float pieceElevation = piece.getElevation();
         boolean pieceMovable = piece.isMovable();
         float [][] pieceModelRotation = piece.getModelRotation();
@@ -1271,8 +1311,6 @@ public class FurnitureController implements Controller {
         retrieveLocalizedData(piece, localizedTags, FurnitureLibrary.FURNITURE_TAGS_PROPERTY);
         Map<String, Object> localizedCategories = new HashMap<String, Object>();
         retrieveLocalizedData(piece, localizedCategories, FurnitureLibrary.FURNITURE_CATEGORY_PROPERTY);
-        // Remove piece from library
-        this.furnitureLibrary.deletePieceOfFurniture(piece);
 
         // Update mandatory not localizable data
         if (model != null) {
@@ -1387,43 +1425,48 @@ public class FurnitureController implements Controller {
           }
         }
        
-        // Create update piece
+        // Create updated piece
+        CatalogPieceOfFurniture updatedPiece;
         if (piece instanceof CatalogDoorOrWindow
             && (doorOrWindow == null || doorOrWindow)) {
           CatalogDoorOrWindow opening = (CatalogDoorOrWindow)piece;
-          piece = new CatalogDoorOrWindow(pieceId, pieceName, pieceDescription, 
+          updatedPiece = new CatalogDoorOrWindow(pieceId, pieceName, pieceDescription, 
               pieceInformation, pieceTags, pieceCreationDate, pieceGrade, 
-              pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, pieceElevation, pieceMovable, 
+              pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, 
+              pieceElevation, pieceDropOnTopElevation, pieceMovable, 
               pieceDoorOrWindowCutOutShape, opening.getWallThickness(), opening.getWallDistance(), opening.getSashes(),
               pieceModelRotation, pieceCreator, pieceResizable, pieceDeformable, pieceTexturable, 
               piecePrice, pieceValueAddedTaxPercentage, pieceCurrency);
         } else if (piece instanceof CatalogLight) {
           CatalogLight light = (CatalogLight)piece;
-          piece = new CatalogLight(pieceId, pieceName, pieceDescription, 
+          updatedPiece = new CatalogLight(pieceId, pieceName, pieceDescription, 
               pieceInformation, pieceTags, pieceCreationDate, pieceGrade, 
-              pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, pieceElevation, pieceMovable, 
+              pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, 
+              pieceElevation, pieceDropOnTopElevation, pieceMovable, 
               light.getLightSources(), pieceStaircaseCutOutShape, 
               pieceModelRotation, pieceCreator, pieceResizable, pieceDeformable, pieceTexturable, 
               piecePrice, pieceValueAddedTaxPercentage, pieceCurrency);
         } else {
           if (doorOrWindow != null && doorOrWindow) {
-            piece = new CatalogDoorOrWindow(pieceId, pieceName, pieceDescription, 
+            updatedPiece = new CatalogDoorOrWindow(pieceId, pieceName, pieceDescription, 
                 pieceInformation, pieceTags, pieceCreationDate, pieceGrade,
-                pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, pieceElevation, pieceMovable, 
+                pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, 
+                pieceElevation, pieceDropOnTopElevation, pieceMovable, 
                 pieceDoorOrWindowCutOutShape, 1, 0, new Sash [0], pieceModelRotation, pieceCreator, 
                 pieceResizable, pieceDeformable, pieceTexturable, 
                 piecePrice, pieceValueAddedTaxPercentage, pieceCurrency);
           } else {
-            piece = new CatalogPieceOfFurniture(pieceId, pieceName, pieceDescription, 
+            updatedPiece = new CatalogPieceOfFurniture(pieceId, pieceName, pieceDescription, 
                 pieceInformation, pieceTags, pieceCreationDate, pieceGrade, 
-                pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, pieceElevation, pieceMovable, 
+                pieceIcon, piecePlanIcon, pieceModel, pieceWidth, pieceDepth, pieceHeight, 
+                pieceElevation, pieceDropOnTopElevation, pieceMovable, 
                 pieceStaircaseCutOutShape, pieceModelRotation, pieceCreator, 
                 pieceResizable, pieceDeformable, pieceTexturable, 
                 piecePrice, pieceValueAddedTaxPercentage, pieceCurrency);
           }
         }
-        new FurnitureCatalog().add(pieceCategory, piece);
-        this.furnitureLibrary.addPieceOfFurniture(piece, index);
+        new FurnitureCatalog().add(pieceCategory, updatedPiece);
+        this.furnitureLibrary.addPieceOfFurniture(updatedPiece, index);
         Set<String> supportedLanguages = new HashSet<String>(this.furnitureLibrary.getSupportedLanguages());
         supportedLanguages.add(this.furnitureLanguageController.getFurnitureLangauge());
         for (String language : supportedLanguages) {
@@ -1431,30 +1474,33 @@ public class FurnitureController implements Controller {
             Object localizedPieceName = localizedNames.get(language);
             if (localizedPieceName != null) {
               this.furnitureLibrary.setPieceOfFurnitureLocalizedData(
-                  piece, language, FurnitureLibrary.FURNITURE_NAME_PROPERTY, localizedPieceName);
+                  updatedPiece, language, FurnitureLibrary.FURNITURE_NAME_PROPERTY, localizedPieceName);
             }
             Object localizedPieceDescription = localizedDescriptions.get(language);
             if (localizedPieceDescription != null) {
               this.furnitureLibrary.setPieceOfFurnitureLocalizedData(
-                  piece, language, FurnitureLibrary.FURNITURE_DESCRIPTION_PROPERTY, localizedPieceDescription);
+                  updatedPiece, language, FurnitureLibrary.FURNITURE_DESCRIPTION_PROPERTY, localizedPieceDescription);
             }          
             Object localizedPieceInformation = localizedInformation.get(language);
             if (localizedPieceInformation != null) {
               this.furnitureLibrary.setPieceOfFurnitureLocalizedData(
-                  piece, language, FurnitureLibrary.FURNITURE_INFORMATION_PROPERTY, localizedPieceInformation);
+                  updatedPiece, language, FurnitureLibrary.FURNITURE_INFORMATION_PROPERTY, localizedPieceInformation);
             }          
             Object localizedPieceTags = localizedTags.get(language);
             if (localizedPieceTags != null) {
               this.furnitureLibrary.setPieceOfFurnitureLocalizedData(
-                  piece, language, FurnitureLibrary.FURNITURE_TAGS_PROPERTY, localizedPieceTags);
+                  updatedPiece, language, FurnitureLibrary.FURNITURE_TAGS_PROPERTY, localizedPieceTags);
             }          
             Object localizedPieceCategory = localizedCategories.get(language);
             if (localizedPieceCategory != null) {
               this.furnitureLibrary.setPieceOfFurnitureLocalizedData(
-                  piece, language, FurnitureLibrary.FURNITURE_CATEGORY_PROPERTY, localizedPieceCategory);
+                  updatedPiece, language, FurnitureLibrary.FURNITURE_CATEGORY_PROPERTY, localizedPieceCategory);
             }
           }
         }
+        
+        // Remove old piece from library
+        this.furnitureLibrary.deletePieceOfFurniture(piece);
       }
     }
   }
